import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Spin, notification, Modal } from 'antd'

import Api from '@/api'
import asyncComponent from '@/utils/asyncComponent'
import UtilsDM from '@/utils/utils-datamanage.js'
import MKEmitter from '@/utils/events.js'
import CalendarBoard from './board'
import './index.scss'

const NormalHeader = asyncComponent(() => import('@/tabviews/custom/components/share/normalheader'))
const MainSearch = asyncComponent(() => import('@/tabviews/zshare/topSearch'))
const PopupButton = asyncComponent(() => import('@/tabviews/zshare/actionList/popupbutton'))

class NormalCalendar extends Component {
  static propTpyes = {
    config: PropTypes.object,        // 组件配置信息
  }

  state = {
    BID: '',                   // 上级ID
    config: null,              // 图表配置信息
    loading: false,            // 数据加载状态
    data: [],                // 数据
    search: '',
    year: new Date().getFullYear(),
    BData: ''
  }

  loaded = false

  /**
   * @description 初始化处理
   */
  UNSAFE_componentWillMount () {
    const { config } = this.props
    let _config = fromJS(config).toJS()
    let BData = ''

    if (_config.setting.supModule) {
      BData = window.GLOB.CacheData.get(_config.setting.supModule)
    } else {
      BData = window.GLOB.CacheData.get(_config.$pageId)
    }

    if (_config.setting.interType === 'system') {
      if (/@mk_year@/ig.test(_config.setting.dataresource) || /@mk_year@/ig.test(_config.setting.customScript)) {
        _config.setting.$re_year = true
      }
    } else {
      _config.setting.$re_year = true
    }

    if (_config.wrap.colorField && (!_config.wrap.signs || _config.wrap.signs.length === 0)) {
      _config.wrap.colorField = ''
    }

    if (_config.action[0] && _config.action[0].config && _config.action[0].config.enabled) {
      _config.setting.linkbtn = _config.action[0].uuid
    } else {
      _config.action = []
    }

    this.setState({
      BID: BData ? (BData.$BID || '') : '',
      BData: BData,
      config: _config,
      arr_field: _config.columns.map(col => col.field).join(','),
      search: _config.$searches
    }, () => {
      if (_config.setting.onload === 'true') {
        setTimeout(() => {
          this.loadData()
        }, _config.setting.delay || 0)
      }
    })
  }

  componentDidMount () {
    const { config } = this.state

    MKEmitter.addListener('reloadData', this.reloadData)
    MKEmitter.addListener('resetSelectLine', this.resetParentParam)
    MKEmitter.addListener('refreshByButtonResult', this.refreshByButtonResult)

    if (config.setting.useMSearch) {
      MKEmitter.addListener('searchRefresh', this.searchRefresh)
    }

    if (config.$cache && !this.loaded) {
      Api.getLCacheConfig(config.uuid).then(res => {
        if (!res.data || this.loaded) return

        let data = []
        res.data.forEach((item, index) => {
          item.key = index
          item.$$uuid = item[config.setting.primaryKey] || ''

          let pass = this.resetLine(item)

          if (pass) {
            data.push(item)
          }
        })

        this.setState({data: data})
      })
    }
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.state), fromJS(nextState))
  }

  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('reloadData', this.reloadData)
    MKEmitter.removeListener('searchRefresh', this.searchRefresh)
    MKEmitter.removeListener('resetSelectLine', this.resetParentParam)
    MKEmitter.removeListener('refreshByButtonResult', this.refreshByButtonResult)
  }

  searchRefresh = (searchId) => {
    const { config } = this.state

    if (config.$searchId !== searchId) return
    
    this.setState({}, () => {
      this.loadData()
    })
  }

  /**
   * @description 按钮执行完成后页面刷新
   * @param {*} menuId     // 菜单Id
   * @param {*} position   // 刷新位置
   * @param {*} btn        // 执行的按钮
   */
  refreshByButtonResult = (menuId, position, btn) => {
    const { config, BID } = this.state

    if (config.uuid !== menuId) return

    if (position === 'mainline' && config.setting.supModule && BID) { // 刷新源组件时，附带刷新上级行与当前组件
      MKEmitter.emit('reloadData', config.setting.supModule, BID)
    } else {
      this.loadData()
    }
  }

  resetParentParam = (MenuID, id, data) => {
    const { config } = this.state

    if (!config.setting.supModule || config.setting.supModule !== MenuID) return
    if (id !== this.state.BID || id !== '') {
      this.setState({ BID: id, BData: data }, () => {
        this.loadData()
      })
    }
  }

  reloadData = (menuId) => {
    const { config } = this.state

    if (config.uuid !== menuId) return

    this.loadData()
  }

  async loadData () {
    const { config, arr_field, BID, search, year } = this.state
    
    if (config.setting.supModule && !BID) { // BID 不存在时，不做查询
      this.setState({
        data: [],
      })
      this.loaded = true
      return
    }

    let searches = fromJS(search).toJS()
    if (config.setting.useMSearch) { // 主表搜索条件
      let mainSearch = window.GLOB.SearchBox.get(config.$searchId) || []
      let keys = config.$s_keys || []
      mainSearch.forEach(item => {
        if (keys.includes(item.key.toLowerCase())) return

        searches.push(item)
      })
    }

    if (config.$s_req && searches.filter(item => item.required && item.value === '').length > 0) {
      return
    }

    this.setState({
      loading: true
    })

    let _orderBy = config.setting.order || ''
    let param = UtilsDM.getQueryDataParams(config.setting, arr_field, searches, _orderBy, 1, 99999, BID, '', year)

    let result = await Api.genericInterface(param)
    if (result.status) {
      this.loaded = true
      if (config.$cache && config.setting.onload !== 'false') {
        Api.writeCacheConfig(config.uuid, result.data || [])
      }

      let data = []
      result.data.forEach((item, index) => {
        item.key = index
        item.$$uuid = item[config.setting.primaryKey] || ''
        item.$$BID = BID || ''

        let pass = this.resetLine(item)

        if (pass) {
          data.push(item)
        }
      })

      this.setState({
        data: data,
        loading: false
      })
      
      if (result.message) {
        if (result.ErrCode === 'Y') {
          Modal.success({
            title: result.message
          })
        } else if (result.ErrCode === 'S') {
          notification.success({
            top: 92,
            message: result.message,
            duration: 2
          })
        }
      }
    } else {
      this.setState({
        loading: false
      })
      
      if (!result.message) return
      if (result.ErrCode === 'N') {
        Modal.error({
          title: result.message,
        })
      } else if (result.ErrCode !== '-2') {
        notification.error({
          top: 92,
          message: result.message,
          duration: 10
        })
      }
    }
  }

  resetLine = (item) => {
    const { config: { wrap } } = this.state

    let startTime = item[wrap.timeField] || ''
    let endTime = item[wrap.endField || wrap.timeField] || ''

    item.$message = item[wrap.remarkField]

    if (!/^(1|2)\d{3}(-|\/)\d{2}(-|\/)\d{2}/.test(startTime)) return false
    if (!/^(1|2)\d{3}(-|\/)\d{2}(-|\/)\d{2}/.test(endTime)) return false
    if (!item.$message) return false

    if (wrap.colorField) {
      let sign = item[wrap.colorField] || ''

      wrap.signs.forEach(cell => {
        if (cell.sign !== sign) return

        item.$style = cell.style
        item.$color = cell.background
        item.$level = cell.$index
      })
    }

    let equal = endTime.substr(0, 4) === startTime.substr(0, 4)

    item.$startM = +(startTime.substr(0, 4) + startTime.substr(5, 2))
    item.$endM = +(endTime.substr(0, 4) + endTime.substr(5, 2))
    item.$start = +(startTime.substr(0, 4) + startTime.substr(5, 2) + startTime.substr(8, 2))
    item.$end = +(endTime.substr(0, 4) + endTime.substr(5, 2) + endTime.substr(8, 2))
    item.$startTime = equal ? `${startTime.substr(5, 2)}-${startTime.substr(8, 2)}` : `${startTime.substr(0, 4)}-${startTime.substr(5, 2)}-${startTime.substr(8, 2)}`
    item.$endTime = equal ?  `${endTime.substr(5, 2)}-${endTime.substr(8, 2)}` : `${endTime.substr(0, 4)}-${endTime.substr(5, 2)}-${endTime.substr(8, 2)}`
    
    return true
  }

  refreshbysearch = (searches) => {
    this.setState({
      search: searches
    }, () => {
      this.loadData()
    })
  }

  yearChange = (value) => {
    const { config } = this.state

    if (!config.setting.$re_year) return

    this.setState({
      year: value
    }, () => {
      this.loadData()
    })
  }

  render() {
    const { config, loading, data, BID, BData } = this.state

    return (
      <div className="normal-calendar-box" id={'anchor' + config.uuid} style={{...config.style}}>
        {loading ?
          <div className="loading-mask">
            <div className="ant-spin-blur"></div>
            <Spin />
          </div> : null
        }
        <NormalHeader config={config} />
        {config.search.length ?
          <MainSearch BID={BID} config={config} refreshdata={this.refreshbysearch}/> : null
        }
        {config.action[0] ? 
          <PopupButton disabled={false} BID={BID} btn={config.action[0]} BData={BData} setting={config.setting} selectedData={[]}/> : null
        }
        <CalendarBoard config={config} data={data} yearChange={this.yearChange}/>
      </div>
    )
  }
}

export default NormalCalendar